from blueness import module
from blue_geo import NAME
from blue_geo.catalog.generic import GenericCatalog
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


class UkraineTimemapCatalog(GenericCatalog):
    name = "ukraine_timemap"

    url = {
        "": "https://ukraine.bellingcat.com/",
        "dataset": "https://bellingcat-embeds.ams3.cdn.digitaloceanspaces.com/production/ukr/timemap/api.json",
        "github": "https://github.com/bellingcat/ukraine-timemap",
    }



from abcli import file
import os
from abcli.plugins.README import build as build_README
from blue_geo.watch.README import items as watch_items
from blue_geo import NAME, VERSION, ICON, REPO_NAME


features = {
    "copernicus": {
        "description": "catalog: [Copernicus Data Space Ecosystem - Europe's eyes on Earth](https://dataspace.copernicus.eu/)",
        "icon": "🧊",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/copernicus.jpg?raw=true",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog/copernicus",
    },
    "firms-area": {
        "description": "catalog: Fire Information for Resource Management System ([FIRMS](https://firms.modaps.eosdis.nasa.gov)).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/datacube-firms_area.jpg",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/catalog/firms",
    },
    "EarthSearch": {
        "description": "catalog: [Earth Search by Element 84 (earth-search-aws)](https://stacindex.org/catalogs/earth-search#/).",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/viewer-aws-element84-com.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/catalog/EarthSearch",
    },
    "ukraine-timemap": {
        "description": "catalog: [Bellingcat](https://www.bellingcat.com/) [Civilian Harm in Ukraine TimeMap](https://github.com/bellingcat/ukraine-timemap) dataset, available through [this UI](https://ukraine.bellingcat.com/) and [this API](https://bellingcat-embeds.ams3.cdn.digitaloceanspaces.com/production/ukr/timemap/api.json).",
        "icon": "🇺🇦",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/nbs/ukraine-timemap/QGIS.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/catalog/ukraine_timemap",
    },
    "vancouver-watching": {
        "description": "catalog: Vancouver watching with AI, last build: [🔗](https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif).",
        "icon": "🌈",
        "thumbnail": "https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif?raw=true",
        "url": "https://github.com/kamangir/Vancouver-Watching",
    },
    "QGIS": {
        "description": "an AI terraform for [QGIS](https://www.qgis.org/).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/QGIS.jpg",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/QGIS/README.md",
    },
    "geo-watch": {
        "description": "watching targets through `@geo`.",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/blue-geo-watch.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/watch",
    },
    "template": {
        "description": "",
        "icon": "",
        "thumbnail": "",
        "url": "",
    },
}


items = [
    "{}[`{}`]({}) [![image]({})]({}) {}".format(
        details["icon"],
        feature,
        details["url"],
        details["thumbnail"],
        details["url"],
        details["description"],
    )
    for feature, details in features.items()
    if feature != "template"
]


def build():
    return all(
        build_README(
            items=items,
            cols=cols,
            path=os.path.join(file.path(__file__), suffix),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
        )
        for items, cols, suffix in zip(
            [[], [], watch_items, items],
            [3, 3, 2, 3],
            ["catalog/copernicus", "catalog/EarthSearch", "watch", ".."],
        )
    )



from typing import List, Dict
import glob
from tqdm import tqdm
import numpy as np
from blueness import module
from abcli import string
from abcli import file
from abcli.plugins.graphics.signature import add_signature
from abcli.plugins.graphics.gif import generate_animated_gif
from abcli.modules.host import signature as host_signature
from abcli.modules.objects import signature as header
from abcli.plugins.metadata import post_to_object
from abcli.modules import objects
from blue_geo import NAME, VERSION
from blue_geo import NAME as BLUE_GEO_NAME
from notebooks_and_scripts import NAME as NBS_NAME, VERSION as NBS_VERSION
from blue_geo.watch.workflow.common import load_watch
from blue_geo.logger import logger


NAME = module.name(__file__, NAME)


def reduce_function(
    query_object_name: str,
    suffix: str,
    object_name: str,
    content_threshold: float = 0.5,
) -> bool:
    success, target, list_of_files = load_watch(object_name)
    if not success:
        return success

    logger.info(
        "{}.reduce {}/{} @ {} -{} file(s)-> {}".format(
            NAME,
            query_object_name,
            suffix,
            target,
            len(list_of_files),
            object_name,
        )
    )

    logger.info("loading metadata ...")
    frame_metadata: Dict[str, Dict] = {}
    bad_metadata: List[str] = []
    for filename in tqdm(
        glob.glob(
            objects.path_of(
                "metadata-*.yaml",
                object_name,
            )
        )
    ):
        success, metadata_ = file.load_yaml(filename)
        if not success:
            bad_metadata.append(filename)
            continue

        if not "filename" in metadata_.get("map", {}):
            logger.info('no "filename" in metadata["map"]: {}.'.format(filename))
            bad_metadata.append(file.name_and_extension(filename))
            continue

        frame_metadata[metadata_["map"]["filename"]] = metadata_
    if bad_metadata:
        logger.info("bad metadata: {}.".format(", ".join(bad_metadata)))

    bad_frames: List[str] = []
    low_content_frames: List[str] = []
    list_of_frames: List[str] = []
    for index, filename in enumerate(list_of_files):
        frame_metadata.setdefault(file.name_and_extension(filename), {})

        datacube_id = (
            frame_metadata[file.name_and_extension(filename)]
            .get("map", {})
            .get("datacube_id", "unknown")
        )

        success, frame = file.load_image(filename)
        if not success:
            bad_frames.append(file.name_and_extension(filename))
            continue

        frame_fp32 = frame.astype(np.float32).flatten() / 255.0
        frame_fp32 = frame_fp32[frame_fp32 > 0.0]
        frame_fp32 = frame_fp32[frame_fp32 < 1.0]
        frame_content_ratio = len(frame_fp32) / frame.size

        frame_has_content = bool(frame_content_ratio > content_threshold)
        logger.info(
            "{} {} / {} @ {}: content={:.03f} <?> {:.03f}".format(
                "✅" if frame_has_content else "🛑",
                datacube_id,
                file.name_and_extension(filename),
                string.pretty_shape_of_matrix(frame),
                frame_content_ratio,
                content_threshold,
            )
        )

        frame_metadata[file.name_and_extension(filename)].update(
            {
                "content": float(frame_content_ratio),
                "has_content": bool(frame_has_content),
                "shape": list(frame.shape),
            }
        )

        if not frame_has_content:
            low_content_frames.append(file.name_and_extension(filename))
            continue

        frame = add_signature(
            frame,
            header=[
                " | ".join(
                    header(
                        "{} / {}".format(
                            datacube_id,
                            file.name_and_extension(filename),
                        ),
                        object_name,
                    )
                    + [
                        "{:05.1f}%".format(frame_content_ratio * 100.0),
                        "{:03d} / {:03d}".format(index, len(list_of_files)),
                    ]
                ),
            ],
            footer=[
                str(target),
                " | ".join(
                    [f"{BLUE_GEO_NAME}.{VERSION}"]
                    + [f"{NBS_NAME}.{NBS_VERSION}"]
                    + host_signature()
                ),
            ],
            word_wrap=False,
        )

        frame_filename = file.set_extension(filename, "png")
        if not file.save_image(frame_filename, frame):
            bad_frames.append(frame_filename)
        else:
            list_of_frames.append(frame_filename)
    if bad_frames:
        logger.error("{} bad frame(s).".format(len(bad_frames)))

    if not post_to_object(
        object_name,
        "reduce",
        {
            "bad_frames": bad_frames,
            "bad_metadata": bad_metadata,
            "frames": frame_metadata,
            "low_content_frames": low_content_frames,
            "list_of_files": [
                file.name_and_extension(filename) for filename in list_of_files
            ],
            "target": target.__dict__,
        },
    ):
        return False

    return all(
        generate_animated_gif(
            list_of_frames,
            objects.path_of(
                "{}{}.gif".format(
                    object_name,
                    f"-{scale}X" if scale != 1 else "",
                ),
                object_name,
            ),
            frame_duration=1000,
            scale=scale,
        )
        for scale in [1, 2]
    )



from blue_geo.catalog.generic.stac.classes import STACCatalog


class CopernicusCatalog(STACCatalog):
    name = "copernicus"

    url = {
        "": "https://dataspace.copernicus.eu/",
        "api": "https://catalogue.dataspace.copernicus.eu/stac",
        "aws_access": "https://documentation.dataspace.copernicus.eu/APIs/S3.html",
        "docs": "https://documentation.dataspace.copernicus.eu/APIs/STAC.html",
    }



from typing import List, Dict


class GenericCatalog:
    name: str = "generic"

    url: Dict[str, str] = {
        "keyword": "url",
    }

    def __init__(self):
        pass

    def get_list_of_collections(self) -> List[str]:
        return []


class VoidCatalog(GenericCatalog):
    name = "void"



from typing import Any, Tuple, List
import boto3
from blue_geo import env
from blue_geo.catalog.copernicus.classes import CopernicusCatalog
from blue_geo.catalog.generic.generic.stac import STACDatacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo.logger import logger


class CopernicusSentinel2Datacube(STACDatacube):
    catalog = CopernicusCatalog()

    collection = "SENTINEL-2"

    name = "sentinel_2"

    def get_bucket(self, verbose: bool = False) -> Tuple[bool, Any, str]:
        try:
            href = (
                self.metadata["Item"]
                .assets["PRODUCT"]
                .extra_fields.get("alternate")["s3"]["href"]
            )
        except:
            return False, None, ""

        bucket_name, s3_prefix = href.split("/", 2)[1:3]

        if verbose:
            logger.info(f"href: {href}")
            logger.info(f"bucket_name: {bucket_name}")
            logger.info(f"s3_prefix: {s3_prefix}")

        s3 = boto3.resource(
            "s3",
            endpoint_url="https://eodata.dataspace.copernicus.eu",
            aws_access_key_id=env.COPERNICUS_AWS_ACCESS_KEY_ID,
            aws_secret_access_key=env.COPERNICUS_AWS_SECRET_ACCESS_KEY,
            region_name="default",
        )

        bucket = s3.Bucket(bucket_name)

        return True, bucket, s3_prefix

    def ingest_filename(
        self,
        filename: str,
        overwrite: bool = False,
        verbose: bool = False,
    ) -> bool:
        if super().ingest_filename(filename, overwrite, verbose):
            return True

        success, bucket, s3_prefix = self.get_bucket(verbose)
        if not success:
            return success

        item_filename = self.full_filename(filename)

        try:
            bucket.download_file(f"{s3_prefix}/{filename}", item_filename)
        except Exception as e:
            logger.error(e)
            return False

        return True

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        success, bucket, s3_prefix = self.get_bucket()
        if not success:
            return []

        list_of_items = bucket.objects.filter(Prefix=s3_prefix)

        list_of_files: List[str] = []
        quick_found = False
        for item in list_of_items:
            item_filename = item.key.split(f"{s3_prefix}/", 1)[1]

            includes, quick_found = scope.includes(
                item_filename, item.size, verbose, quick_found
            )
            if includes:
                list_of_files.append(item_filename)

        return list_of_files



from typing import List
from blueness import module
from abcli.modules.host import shell
from blue_geo import NAME
from blue_geo.catalog.EarthSearch.classes import EarthSearchCatalog
from blue_geo.catalog.generic.generic.stac import STACDatacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


class EarthSearchSentinel2L1CDatacube(STACDatacube):
    catalog = EarthSearchCatalog()

    collection = "sentinel-2-l1c"

    name = "sentinel_2_l1c"

    s3_prefix = "s3://sentinel-s2-l1c/tiles"

    def ingest_filename(
        self,
        filename: str,
        overwrite: bool = False,
        verbose: bool = False,
    ) -> bool:
        if super().ingest_filename(filename, overwrite, verbose):
            return True

        # https://registry.opendata.aws/sentinel-2/
        return shell(
            "aws s3 cp --request-payer requester {}/{} {}".format(
                self.s3_prefix,
                filename.replace("_", "/"),
                self.full_filename(filename),
            )
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        list_of_items = [asset.href for asset in self.metadata["Item"].assets.values()]

        list_of_files: List[str] = []
        quick_found = False
        for item in list_of_items:
            item_filename = item.split(f"{self.s3_prefix}/", 1)[1].replace("/", "_")

            includes, quick_found = scope.includes(
                item_filename, -1, verbose, quick_found
            )
            if includes:
                list_of_files.append(item_filename)

        return list_of_files



from typing import Tuple, Dict, Any, List
from datetime import datetime, timedelta
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
from abcli import file
from abcli.modules import objects
from abcli.plugins import metadata
from abcli.plugins.metadata import post_to_object
from blue_geo.catalog.firms import FirmsCatalog
from blue_geo.catalog.generic import GenericDatacube
from blue_geo.catalog.firms.area.enums import Area, Source
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo import env
from blue_geo.logger import logger


class FirmsAreaDatacube(GenericDatacube):
    name = "area"

    catalog = FirmsCatalog()

    query_args = {
        "area": {
            "default": Area.default().name,
            "help": "|".join(Area.values()),
        },
        "date": {
            "default": (datetime.now() - timedelta(days=5)).strftime("%Y-%m-%d"),
            "help": "<yyyy-mm-dd>",
        },
        "depth": {
            "type": int,
            "default": 1,
            "help": "1..10",
        },
        "source": {
            "default": Source.default().name,
            "help": "|".join(Source.values()),
        },
    }

    QGIS_template = env.BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA

    def __init__(
        self,
        datacube_id: str = "",
        source: Source = Source.default(),
        area: Area = Area.default(),
        date: str = "",
        depth: int = 1,
    ):
        super().__init__(datacube_id)

        self.url_prefix = "https://firms.modaps.eosdis.nasa.gov/api/area"
        self.map_key = env.FIRMS_MAP_KEY

        if datacube_id:
            success, args = FirmsAreaDatacube.parse_datacube_id(datacube_id)
            assert success

            area = args["area"]
            source = args["source"]
            depth = args["depth"]
            date = args["date"]

        self.area: Area = area
        self.source: Source = source

        self.depth: int = depth

        self.date: str = (
            date if date else (datetime.now() - timedelta(days=5)).strftime("%Y-%m-%d")
        )

        self.build_datacube_id()

    def build_datacube_id(self):
        self.datacube_id = "datacube-{}-{}-{}-{}-{}-{}".format(
            self.catalog.name,
            self.name,
            self.area.name.lower(),
            self.source.name,
            self.date,
            self.depth,
        )

    @property
    def description(self) -> str:
        return "{}, area:{}, source:{} ({})".format(
            super().description,
            self.area.name.lower(),
            self.source.name,
            self.source.description,
        )

    def ingest(
        self,
        dryrun: bool = False,
        overwrite: bool = False,
        scope: str = "metadata",
    ) -> Tuple[
        bool,
        gpd.GeoDataFrame,
    ]:
        success, _ = super().ingest(dryrun, overwrite, scope)
        if not success:
            return success, gpd.GeoDataFrame()

        csv_filename = objects.path_of("firms_area.csv", self.datacube_id, create=True)
        if not file.download(
            self.ingest_url(),
            csv_filename,
        ):
            return False, gpd.GeoDataFrame()

        data = pd.read_csv(csv_filename)
        if data.empty:
            return False, gpd.GeoDataFrame()

        logger.info(f"loaded {len(data):,} point(s).")

        gdf = gpd.GeoDataFrame(
            data,
            geometry=[
                Point(xy)
                for xy in zip(
                    data.longitude,
                    data.latitude,
                )
            ],
            crs="EPSG:4326",  # WGS84
        )

        if not file.save_geojson(
            objects.path_of("firms_area.geojson", self.datacube_id),
            gdf,
        ):
            return False, gdf

        if not metadata.post_to_object(
            self.datacube_id,
            "datacube",
            {
                "id": self.datacube_id,
                "source": self.source.name,
                "area": self.area.name,
                "date": self.date,
                "depth": self.depth,
                "len": len(gdf),
            },
        ):
            return False, gdf

        return True, gdf

    def ingest_url(self, html: bool = False) -> str:
        return "{}/{}/{}/{}/{}/{}/{}".format(
            self.url_prefix,
            "html" if html else "csv",
            self.map_key,
            self.source.name,
            self.area.name.lower(),
            self.depth,  # day_range
            self.date,
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        return [
            f"firms_area.{extension}"
            for extension in [
                "geojson",
                "csv",
            ]
        ]

    @classmethod
    def parse_datacube_id(cls, datacube_id: str) -> Tuple[
        bool,
        Dict[str, Any],
    ]:
        success, _ = super().parse_datacube_id(datacube_id)
        if not success:
            return False, {}

        # datacube-firm-area-<area>-<source>-yyyy-mm-dd-depth
        # datacube-firms-area-world-MODIS_NRT-2024-07-20-1
        segments = datacube_id.split("-")
        if len(segments) < 9:
            return False, {}

        if segments[2] != cls.name:
            return False, {}

        area_str = segments[3]
        if area_str not in Area.values():
            return False, {}

        source_str = segments[4]
        if source_str not in Source.values():
            return False, {}

        date = "{}-{}-{}".format(segments[5], segments[6], segments[7])

        depth_str = segments[8]
        if not depth_str.isdigit():
            return False, {}

        return (
            True,
            {
                "area": Area[area_str.upper()],
                "source": Source[source_str],
                "date": date,
                "depth": int(depth_str),
            },
        )

    @classmethod
    def query(cls, object_name: str, **kwargs) -> bool:
        logger.info(f"🔎 {cls.__name__}.query -> {object_name}")

        datacube = cls(**kwargs)

        logger.info(f"🧊 {datacube.description}")

        return post_to_object(
            object_name,
            "datacube_id",
            [datacube.datacube_id],
        )



import os
from typing import List, Union
import argparse
from blueness import module
from abcli import file
from blue_geo import NAME, VERSION
from blue_geo.watch.targets.classes import TargetList, Target
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

list_of_tasks = "get|save"

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--what",
    default="",
    type=str,
    help="catalog|collection|exists|list|query_args",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--log",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--target_name",
    type=str,
)
parser.add_argument(
    "--object_name",
    type=str,
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

target_list = TargetList(os.path.join(file.path(__file__), "../targets.yaml"))

target = target_list.targets.get(args.target_name, Target())

success = args.task in list_of_tasks
if args.task == "get":
    output: Union[str, List[str]] = []

    if args.what == "query_args":
        output = target.query_args_as_str()
    elif args.what == "catalog":
        output = target.catalog
    elif args.what == "collection":
        output = target.collection
    elif args.what == "exists":
        output = str(bool(target.name))
    elif args.what == "list":
        output = list(target_list.targets.keys())
    else:
        success = False

    if args.count != -1 and isinstance(output, list):
        output = output[: args.count]

    if success:
        if args.log:
            logger.info(
                "{:,} {}(s): {}".format(
                    len(output),
                    args.what,
                    delim.join(output),
                )
                if isinstance(output, list)
                else output
            )
        else:
            print(delim.join(output) if isinstance(output, list) else output)
elif args.task == "save":
    success = target.save(args.object_name)
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from abcli import string
from blue_geo import REPO_NAME

url_prefix = "https://kamangir-public.s3.ca-central-1.amazonaws.com"


items = [
    " ".join(
        [
            f"![image]({url_prefix}/{object_name}/{object_name}.gif?raw=true&random={string.random_()})",
            f"{description}, object: [`{object_name}`]({url_prefix}/{object_name}.tar.gz), [gif]({url_prefix}/{object_name}/{object_name}.gif).",
        ]
    )
    for object_name, description in zip(
        [
            "test_blue_geo_watch",
            "geo-watch-2024-08-31-chilcotin-c",
        ],
        [
            f"[![bashtest](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml/badge.svg)](https://github.com/kamangir/{REPO_NAME}/actions/workflows/bashtest.yml)",
            "1️⃣  L1C and L2A mixed, `2024-07-30/2024-08-09`, [dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-199-11f9b5497ef0)",
        ],
    )
] + [
    " ".join(
        [
            f"![image]({url_prefix}/{object_name}/{object_name}-2X.gif?raw=true&random={string.random_()})",
            f"{description}, object: [`{object_name}`]({url_prefix}/{object_name}.tar.gz), [gif]({url_prefix}/{object_name}/{object_name}.gif).",
        ]
    )
    for object_name, description in zip(
        [
            "geo-watch-2024-09-01-chilcotin-a",
            "geo-watch-2024-09-01-chilcotin-c",
        ],
        [
            "1️⃣ [dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-201-d64e9bb3716b)",
            "1️⃣ [dev notes](https://arash-kamangir.medium.com/%EF%B8%8F-conversations-with-ai-202-d59ba811398b)",
        ],
    )
]



import argparse
from blueness import module
from blue_geo import NAME, VERSION
from blue_geo.catalog.default import add_default_arguments
from blue_geo.catalog.firms.area.enums import Area, Source
from blue_geo.catalog.firms.area.classes import FirmsAreaDatacube
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="query",
)
add_default_arguments(FirmsAreaDatacube.query_args, parser)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
args = parser.parse_args()


success = False
if args.task == "query":
    success = FirmsAreaDatacube.query(
        object_name=args.object_name,
        area=Area[args.area],
        source=Source[args.source],
        depth=args.depth,
        date=args.date,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List, Type
from blue_geo.catalog.generic import (
    GenericCatalog,
    VoidCatalog,
    GenericDatacube,
    VoidDatacube,
)
from blue_geo.catalog.copernicus import CopernicusCatalog, CopernicusSentinel2Datacube
from blue_geo.catalog.EarthSearch import (
    EarthSearchCatalog,
    EarthSearchSentinel2L1CDatacube,
)
from blue_geo.catalog.firms import FirmsCatalog
from blue_geo.catalog.firms.area import FirmsAreaDatacube
from blue_geo.catalog.ukraine_timemap import (
    UkraineTimemapCatalog,
    UkraineTimemapDatacube,
)

list_of_catalog_classes: List[Type[GenericCatalog]] = [
    GenericCatalog,
    CopernicusCatalog,
    EarthSearchCatalog,
    FirmsCatalog,
    UkraineTimemapCatalog,
]

list_of_catalogs: List[str] = sorted(
    [catalog_class.name for catalog_class in list_of_catalog_classes]
)

list_of_datacube_classes: List[Type[GenericDatacube]] = [
    GenericDatacube,
    FirmsAreaDatacube,
    UkraineTimemapDatacube,
    CopernicusSentinel2Datacube,
    EarthSearchSentinel2L1CDatacube,
]



import os
from typing import Any, Tuple, Dict, List
from blueness import module
from abcli import file, path
from abcli.modules import objects
from blue_geo import NAME
from abcli.plugins.metadata import post_to_object
from blue_geo.catalog.generic.classes import GenericCatalog, VoidCatalog
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


class GenericDatacube:
    name = "generic"

    catalog = GenericCatalog()

    datacube_id = ""

    metadata: Any = None

    query_args: Dict[str, Dict] = {
        "arg": {
            "default": "value",
            "help": "<value>",
        },
    }

    QGIS_template = "unknown-template"

    def __init__(self, datacube_id: str = ""):
        self.datacube_id = datacube_id
        if self.datacube_id:
            self.update_metadata()

    @property
    def description(self) -> str:
        return "{}.{}@{}[{}]:".format(
            NAME,
            self.__class__.__name__,
            self.catalog.name,
            self.datacube_id,
        )

    def full_filename(self, filename: str) -> str:
        return objects.path_of(filename, self.datacube_id, create=True)

    def ingest(
        self,
        dryrun: bool = False,
        overwrite: bool = False,
        scope: str = "metadata",
    ) -> Tuple[bool, Any]:
        logger.info(
            "{}.{}.ingest({}): {} @ {}".format(
                NAME,
                self.__class__.__name__,
                ",".join(
                    [
                        item
                        for item in ["dryrun" if dryrun else ""]
                        + ["overwrite" if overwrite else ""]
                        if item
                    ]
                ),
                scope,
                self.datacube_id,
            )
        )

        return True, None

    # returns True if ingest is complete.
    def ingest_filename(
        self,
        filename: str,
        overwrite: bool = False,
        verbose: bool = False,
    ) -> bool:
        item_filename = self.full_filename(filename)

        assert path.create(file.path(item_filename))

        if item_filename.endswith(os.sep):
            return True

        if not overwrite and file.exist(item_filename):
            logger.info(f"✅ {item_filename}")
            return True

        logger.info("ingesting {} ...".format(filename))

        return False

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        return []

    @classmethod
    def parse_datacube_id(cls, datacube_id: str) -> Tuple[
        bool,
        Dict[str, Any],
    ]:
        # datacube-<catalog-name>-<args>
        segments = datacube_id.split("-") + ["", ""]

        return (
            segments[0] == "datacube" and segments[1] == cls.catalog.name,
            {},
        )

    @property
    def path(self) -> str:
        return objects.object_path(self.datacube_id)

    @classmethod
    def query(cls, object_name: str) -> bool:
        logger.info(f"🔎 {cls.__name__}.query -> {object_name}")

        return post_to_object(object_name, "datacube_id", [])

    def update_metadata(self, verbose: bool = False) -> bool:
        if verbose:
            logger.info(f"{self.description}.update_metadata()")

        return True


class VoidDatacube(GenericDatacube):
    name = "void"
    catalog = VoidCatalog()



from typing import Tuple
from abcli import string
from blue_geo.logger import logger

raster_suffix = [
    ".jp2",
    ".tif",
    ".tiff",
]


class DatacubeScope:
    special_options = ["all", "metadata", "quick", "raster"]

    help = "|".join(special_options + ["<{}>".format("+".join(raster_suffix))])

    def __init__(self, what: str):
        list_of_what = what.split("+")

        self.all = "all" in list_of_what

        self.metadata = "metadata" in list_of_what

        self.quick = "quick" in list_of_what

        self.raster = "raster" in list_of_what

        self.suffix = [
            item for item in list_of_what if item not in self.special_options
        ]

    def includes(
        self,
        item_filename: str,
        item_size: int = -1,
        verbose: bool = False,
        quick_found: bool = False,
    ) -> Tuple[bool, bool]:
        if self.all:
            return True, quick_found

        if (
            self.metadata
            and item_size <= 10**6
            and not any(item_filename.endswith(suffix) for suffix in raster_suffix)
        ):
            return True, quick_found

        if (
            self.quick
            and not quick_found
            and item_filename.endswith(".jp2")
            and "TCI" in item_filename
        ):
            return True, True

        if self.raster and any(
            item_filename.endswith(suffix) for suffix in raster_suffix
        ):
            return True, quick_found

        if self.suffix and any(
            item_filename.endswith(suffix) for suffix in self.suffix
        ):
            return True, quick_found

        if verbose:
            logger.info(
                "skipped {}: {}".format(
                    string.pretty_bytes(item_size),
                    item_filename,
                )
            )

        return False, quick_found



from blueness import module
from blue_geo import NAME
from blue_geo.catalog.generic import GenericCatalog
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


class FirmsCatalog(GenericCatalog):
    name = "firms"

    url = {
        "": "https://firms.modaps.eosdis.nasa.gov/",
        "area": "https://firms.modaps.eosdis.nasa.gov/api/area/",
        "map_key": "https://firms.modaps.eosdis.nasa.gov/api/map_key/",
    }



from blue_geo import NAME, VERSION, DESCRIPTION, REPO_NAME
from blueness.pypi import setup


setup(
    filename=__file__,
    repo_name=REPO_NAME,
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    packages=[
        NAME,
        f"{NAME}.catalog",
        f"{NAME}.catalog.copernicus",
        f"{NAME}.catalog.copernicus.sentinel_2",
        f"{NAME}.catalog.EarthSearch",
        f"{NAME}.catalog.EarthSearch.sentinel_2_l1c",
        f"{NAME}.catalog.firms",
        f"{NAME}.catalog.firms.area",
        f"{NAME}.catalog.generic",
        f"{NAME}.catalog.generic.generic",
        f"{NAME}.catalog.ukraine_timemap",
        f"{NAME}.catalog.ukraine_timemap.ukraine_timemap",
        f"{NAME}.datacube",
        f"{NAME}.QGIS",
        f"{NAME}.watch",
        f"{NAME}.watch.targets",
        f"{NAME}.watch.workflow",
    ],
    include_package_data=True,
    package_data={
        NAME: [
            "config.env",
            "sample.env",
            ".abcli/**/*.sh",
            "**/*.md",
            "watch/targets.yaml",
        ],
    },
)



from enum import Enum, auto


class Area(Enum):
    EAST = auto()
    NORTH = auto()
    SOUTH = auto()
    WEST = auto()
    WORLD = auto()

    @classmethod
    def default(cls):
        return Area.WORLD

    @classmethod
    def values(cls):
        return [area.name.lower() for area in Area]


class Source(Enum):
    LANDSAT_NRT = (
        auto(),
        "LANDSAT Near Real-Time, Real-Time and Ultra Real-Time [US/Canada only]",
    )
    MODIS_NRT = (
        auto(),
        "MODIS Near Real-Time, Real-Time and Ultra Real-Time",
    )
    MODIS_SP = (
        auto(),
        "(MODIS Standard Processing)",
    )
    VIIRS_NOAA20_NRT = (
        auto(),
        "VIIRS NOAA-20 Near Real-Time, Real-Time and Ultra Real-Time",
    )
    VIIRS_NOAA21_NRT = (
        auto(),
        "VIIRS NOAA-21 Near Real-Time, Real-Time and Ultra Real-Time",
    )
    VIIRS_SNPP_NRT = (
        auto(),
        "VIIRS Suomi-NPP Near Real-Time, Real-Time and Ultra Real-Time",
    )
    VIIRS_SNPP_SP = (
        auto(),
        "VIIRS Suomi-NPP Standard Processing",
    )

    def __init__(self, _, description):
        self.description = description

    @classmethod
    def default(cls):
        return Source.MODIS_NRT

    @classmethod
    def values(cls):
        return [source.name for source in Source]
